#===============================================================
#         Makefile for generating rmodfgr.exe
#
# Targets:
#         rmodfgr.exe, clean
#
# Usage:
#         To build rmodfgr using gfortran (GNU):
#            $ make -f rmodfgr.mk             ! only first target (default)
#         To clean up:
#            $ make -f rmodfgr.mk clean                       ! second target
#         To build rmodfgr using gfortran (GNU) and clean up:
#            $ make -f rmodfgr.mk rmodfgr.exe clean          ! both targets
#         To run the program:
#            $ chmod a+x rmodfgr.exe
#            $ ./rmodfgr.exe
#
#===============================================================

#--------------------
# Root directory
#--------------------
RP = /home/sveni/SI/oalib

#-------------------------
# Source code directories
#-------------------------
r_cpm_dir         = $(RP)/rpress/cpm
r_single_dir      = $(RP)/rpress/cpm/single
r_dhcomp_dir      = $(RP)/rpress/cpm/dhcomp
r_int_dir         = $(RP)/rpress/int
r_rut_dir         = $(RP)/rpress/rut
r_slatec_dir      = $(RP)/rpress/slatec
linu_dir          = $(RP)/rpress/linu

#-----------------------
# Lists of code to link
#-----------------------
r_cpm_list         = INMODELa.o PREDHASa.o GRLOCALa.o\
                     azimiln.o dhsub.o ec.o fsubmodelshell.o\
                     harksi.o harksipart.o liqpart.o\
                     u2test.o grderiv.o\
                     harkderiva.o harkderivb.o liqderiva.o liqderivb.o 
r_single_list      = liqsi.o rmodel.o
r_dhcomp_list      = dhcomp.o
r_int_list         = dpclin.o dlamch.o lsame.o
r_rut_list         = dzeroholo.o\
                     eimult.o rhypsmall.o hptrig.o blanktoend.o putintp.o\
                     datan2dq.o cdsqrt12.o\
                      hyprest.o rtext.o
r_slatec_list      = airy2e.o airy4e.o aizbiz.o
linu_list          = d1mach.o

#--------------------
# Common code
#--------------------
OBJECTS = $(r_cpm_list) $(r_single_list) $(r_dhcomp_list)\
          $(r_int_list) $(r_rut_list) $(r_slatec_list)\
          $(linu_list)

#-----------------------------
# Compilers and compiler flags
#-----------------------------
F90 = /usr/bin/gfortran
FFLAGS  = -fcheck=all -fPIC -ffixed-line-length-132     -fno-align-commons
F_FLAGS =             -fPIC -ffixed-line-length-132 -O3 -fno-align-commons

#------------------
# Compile commands
#------------------
COMPILE.f90 = $(F90) $(FFLAGS) -c
COMPILE_f90 = $(F90) $(F_FLAGS) -c

#---------------
# Link commands
#---------------
LINK.f90 = $(F90) $(FFLAGS) -o
LINK_f90 = $(F90) $(F_FLAGS) -o

#===============================================================

#--------------------
# Link code
#--------------------
rmodfgr.exe: rmodfgr.o dispcurve.o $(OBJECTS)
	$(LINK.f90) rmodfgr.exe rmodfgr.o dispcurve.o $(OBJECTS)

#===============================================================

#--------------------
# Compile code
#--------------------
rmodfgr.o: $(r_cpm_dir)/rmodfgr.f\
                          INMODELa.o
	$(COMPILE.f90) $(r_cpm_dir)/rmodfgr.f

dispcurve.o: $(r_cpm_dir)/dispcurve.f
	$(COMPILE.f90) $(r_cpm_dir)/dispcurve.f

#--------------------
# Compile code
#--------------------
INMODELa.o: $(r_cpm_dir)/INMODELa.f
	$(COMPILE.f90) $(r_cpm_dir)/INMODELa.f

PREDHASa.o: $(r_cpm_dir)/PREDHASa.f
	$(COMPILE.f90) $(r_cpm_dir)/PREDHASa.f

GRLOCALa.o: $(r_cpm_dir)/GRLOCALa.f
	$(COMPILE.f90) $(r_cpm_dir)/GRLOCALa.f

azimiln.o: $(r_cpm_dir)/azimiln.f
	$(COMPILE.f90) $(r_cpm_dir)/azimiln.f

dhsub.o: $(r_cpm_dir)/dhsub.f
	$(COMPILE.f90) $(r_cpm_dir)/dhsub.f

ec.o: $(r_cpm_dir)/ec.f
	$(COMPILE.f90) $(r_cpm_dir)/ec.f

fsubmodelshell.o: $(r_cpm_dir)/fsubmodelshell.f
	$(COMPILE.f90) $(r_cpm_dir)/fsubmodelshell.f

harksi.o: $(r_cpm_dir)/harksi.f
	$(COMPILE.f90) $(r_cpm_dir)/harksi.f

harksipart.o: $(r_cpm_dir)/harksipart.f
	$(COMPILE.f90) $(r_cpm_dir)/harksipart.f

liqpart.o: $(r_cpm_dir)/liqpart.f
	$(COMPILE.f90) $(r_cpm_dir)/liqpart.f

u2test.o: $(r_cpm_dir)/u2test.f
	$(COMPILE.f90) $(r_cpm_dir)/u2test.f

grderiv.o: $(r_cpm_dir)/grderiv.f
	$(COMPILE.f90) $(r_cpm_dir)/grderiv.f

harkderiva.o: $(r_cpm_dir)/harkderiva.f
	$(COMPILE.f90) $(r_cpm_dir)/harkderiva.f

harkderivb.o: $(r_cpm_dir)/harkderivb.f
	$(COMPILE.f90) $(r_cpm_dir)/harkderivb.f

liqderiva.o: $(r_cpm_dir)/liqderiva.f
	$(COMPILE.f90) $(r_cpm_dir)/liqderiva.f

liqderivb.o: $(r_cpm_dir)/liqderivb.f
	$(COMPILE.f90) $(r_cpm_dir)/liqderivb.f

liqsi.o: $(r_single_dir)/liqsi.f
	$(COMPILE.f90) $(r_single_dir)/liqsi.f

rmodel.o: $(r_single_dir)/rmodel.f
	$(COMPILE.f90) $(r_single_dir)/rmodel.f

dhcomp.o: $(r_dhcomp_dir)/dhcomp.f
	$(COMPILE.f90) $(r_dhcomp_dir)/dhcomp.f

dpclin.o: $(r_int_dir)/dpclin.f
	$(COMPILE.f90) $(r_int_dir)/dpclin.f

dlamch.o: $(r_int_dir)/dlamch.f
	$(COMPILE.f90) $(r_int_dir)/dlamch.f

lsame.o: $(r_int_dir)/lsame.f
	$(COMPILE.f90) $(r_int_dir)/lsame.f

dzeroholo.o: $(r_rut_dir)/dzeroholo.f
	$(COMPILE.f90) $(r_rut_dir)/dzeroholo.f

eimult.o: $(r_rut_dir)/eimult.f
	$(COMPILE.f90) $(r_rut_dir)/eimult.f

rhypsmall.o: $(r_rut_dir)/rhypsmall.f
	$(COMPILE.f90) $(r_rut_dir)/rhypsmall.f

hptrig.o: $(r_rut_dir)/hptrig.f
	$(COMPILE.f90) $(r_rut_dir)/hptrig.f

blanktoend.o: $(r_rut_dir)/blanktoend.f
	$(COMPILE.f90) $(r_rut_dir)/blanktoend.f

putintp.o: $(r_rut_dir)/putintp.f
	$(COMPILE.f90) $(r_rut_dir)/putintp.f

datan2dq.o: $(r_rut_dir)/datan2dq.f
	$(COMPILE.f90) $(r_rut_dir)/datan2dq.f

cdsqrt12.o: $(r_rut_dir)/cdsqrt12.f
	$(COMPILE.f90) $(r_rut_dir)/cdsqrt12.f

hyprest.o: $(r_rut_dir)/hyprest.f
	$(COMPILE.f90) $(r_rut_dir)/hyprest.f

rtext.o: $(r_rut_dir)/rtext.f
	$(COMPILE.f90) $(r_rut_dir)/rtext.f

airy2e.o: $(r_slatec_dir)/airy2e.f
	$(COMPILE.f90) $(r_slatec_dir)/airy2e.f

airy4e.o: $(r_slatec_dir)/airy4e.f
	$(COMPILE.f90) $(r_slatec_dir)/airy4e.f

aizbiz.o: $(r_slatec_dir)/aizbiz.f\
                        d1mach.o
	$(COMPILE.f90) $(r_slatec_dir)/aizbiz.f

d1mach.o: $(linu_dir)/d1mach.f90
	$(COMPILE.f90) $(linu_dir)/d1mach.f90

#===============================================================

#--------------------
# Auxiliaries
#--------------------
clean:
	rm -rf *.o *.a *~ *.mod

