#===============================================================
#         Makefile for generating rpress.exe
#
# Targets:
#         rpress.exe, clean
#
# Usage:
#         To build rpress using gfortran (GNU):
#            $ make -f rpress.mk             ! only first target (default)
#         To clean up:
#            $ make -f rpress.mk clean                     ! second target
#         To build rpress using gfortran (GNU) and clean up:
#            $ make -f rpress.mk rpress.exe clean          ! both targets
#         To run the program:
#            $ chmod a+x rpress.exe
#            $ ./rpress.exe
#
#===============================================================

#--------------------
# Root directory
#--------------------
RP = /home/sveni/SI/oalib
RP = /home/arthur/models

#-------------------------
# Source code directories
#-------------------------
r_cpm_dir         = $(RP)/rpress/cpm
r_single_dir      = $(RP)/rpress/cpm/single
r_dhcomp_dir      = $(RP)/rpress/cpm/dhcomp
r_int_dir         = $(RP)/rpress/int
r_bess_dir        = $(RP)/rpress/bess
r_rut_dir         = $(RP)/rpress/rut
r_slatec_dir      = $(RP)/rpress/slatec
linu_dir          = $(RP)/rpress/linu

#-----------------------
# Lists of code to link
#-----------------------
r_cpm_list         = INMODELa.o INRANGEa.o PREDHASa.o INTWORKa.o\
                     rpressint.o terfun.o rsub.o rpbranchint.o branchsub.o\
                     ia0_functions.o\
                     azimiln.o dhsub.o ec.o fsubmodelshell.o\
                     harksi.o harksipart.o liqpart.o\
                     u2test.o utspec.o
r_single_list      = liqsi.o rmodel.o
r_dhcomp_list      = dhcomp.o
r_int_list         = dpclin.o xpolbul.o adintxfi.o trapezfi.o dlamch.o lsame.o
r_bess_list        = besbod.o besrat.o\
                     bessrtra.o
r_rut_list         = euleracc.o\
                     eimult.o rhypsmall.o hptrig.o blanktoend.o putintp.o\
                     datan2dq.o cdsqrt12.o\
                     cfftal.o silvia.o zerodivy.o argseg.o hyprest.o rtext.o
r_slatec_list      = airy2e.o airy4e.o aizbiz.o
linu_list          = d1mach.o

#--------------------
# Common code
#--------------------
OBJECTS = $(r_cpm_list) $(r_single_list) $(r_dhcomp_list)\
          $(r_int_list) $(r_bess_list) $(r_rut_list) $(r_slatec_list)\
          $(linu_list)

#-----------------------------
# Compilers and compiler flags
#-----------------------------
F90 = /usr/bin/gfortran
FFLAGS  = -fcheck=all -fPIC -ffixed-line-length-132     -fno-align-commons
F_FLAGS =             -fPIC -ffixed-line-length-132 -O3 -fno-align-commons

#------------------
# Compile commands
#------------------
COMPILE.f90 = $(F90) $(FFLAGS) -c
COMPILE_f90 = $(F90) $(F_FLAGS) -c

#---------------
# Link commands
#---------------
LINK.f90 = $(F90) $(FFLAGS) -o
LINK_f90 = $(F90) $(F_FLAGS) -o

#===============================================================

#--------------------
# Link code
#--------------------
rpress.exe: rpress.o rpressmain.o $(OBJECTS)
	$(LINK.f90) rpress.exe rpress.o rpressmain.o $(OBJECTS)

#===============================================================

#--------------------
# Compile code
#--------------------
rpress.o: $(r_cpm_dir)/rpress.f\
                          rpressmain.o
	$(COMPILE.f90) $(r_cpm_dir)/rpress.f

rpressmain.o: $(r_cpm_dir)/rpressmain.f\
                          rmodel.o rpressint.o
	$(COMPILE.f90) $(r_cpm_dir)/rpressmain.f

#--------------------
# Compile code
#--------------------
INMODELa.o: $(r_cpm_dir)/INMODELa.f
	$(COMPILE.f90) $(r_cpm_dir)/INMODELa.f

INRANGEa.o: $(r_cpm_dir)/INRANGEa.f
	$(COMPILE.f90) $(r_cpm_dir)/INRANGEa.f

PREDHASa.o: $(r_cpm_dir)/PREDHASa.f
	$(COMPILE.f90) $(r_cpm_dir)/PREDHASa.f

INTWORKa.o: $(r_cpm_dir)/INTWORKa.f
	$(COMPILE.f90) $(r_cpm_dir)/INTWORKa.f

rpressint.o: $(r_cpm_dir)/rpressint.f\
                          INRANGEa.o PREDHASa.o INTWORKa.o
	$(COMPILE.f90) $(r_cpm_dir)/rpressint.f

terfun.o: $(r_cpm_dir)/terfun.f
	$(COMPILE.f90) $(r_cpm_dir)/terfun.f

rsub.o: $(r_cpm_dir)/rsub.f
	$(COMPILE.f90) $(r_cpm_dir)/rsub.f

rpbranchint.o: $(r_cpm_dir)/rpbranchint.f
	$(COMPILE.f90) $(r_cpm_dir)/rpbranchint.f

branchsub.o: $(r_cpm_dir)/branchsub.f
	$(COMPILE.f90) $(r_cpm_dir)/branchsub.f

ia0_functions.o: $(r_cpm_dir)/ia0_functions.f
	$(COMPILE.f90) $(r_cpm_dir)/ia0_functions.f

azimiln.o: $(r_cpm_dir)/azimiln.f
	$(COMPILE.f90) $(r_cpm_dir)/azimiln.f

dhsub.o: $(r_cpm_dir)/dhsub.f
	$(COMPILE.f90) $(r_cpm_dir)/dhsub.f

ec.o: $(r_cpm_dir)/ec.f
	$(COMPILE.f90) $(r_cpm_dir)/ec.f

fsubmodelshell.o: $(r_cpm_dir)/fsubmodelshell.f
	$(COMPILE.f90) $(r_cpm_dir)/fsubmodelshell.f

harksi.o: $(r_cpm_dir)/harksi.f
	$(COMPILE.f90) $(r_cpm_dir)/harksi.f

harksipart.o: $(r_cpm_dir)/harksipart.f
	$(COMPILE.f90) $(r_cpm_dir)/harksipart.f

liqpart.o: $(r_cpm_dir)/liqpart.f
	$(COMPILE.f90) $(r_cpm_dir)/liqpart.f

u2test.o: $(r_cpm_dir)/u2test.f
	$(COMPILE.f90) $(r_cpm_dir)/u2test.f

utspec.o: $(r_cpm_dir)/utspec.f
	$(COMPILE.f90) $(r_cpm_dir)/utspec.f

liqsi.o: $(r_single_dir)/liqsi.f
	$(COMPILE.f90) $(r_single_dir)/liqsi.f

rmodel.o: $(r_single_dir)/rmodel.f\
                          INMODELa.o
	$(COMPILE.f90) $(r_single_dir)/rmodel.f

dhcomp.o: $(r_dhcomp_dir)/dhcomp.f
	$(COMPILE.f90) $(r_dhcomp_dir)/dhcomp.f

dpclin.o: $(r_int_dir)/dpclin.f
	$(COMPILE.f90) $(r_int_dir)/dpclin.f

xpolbul.o: $(r_int_dir)/xpolbul.f
	$(COMPILE.f90) $(r_int_dir)/xpolbul.f

adintxfi.o: $(r_int_dir)/adintxfi.f
	$(COMPILE.f90) $(r_int_dir)/adintxfi.f

trapezfi.o: $(r_int_dir)/trapezfi.f
	$(COMPILE.f90) $(r_int_dir)/trapezfi.f

dlamch.o: $(r_int_dir)/dlamch.f
	$(COMPILE.f90) $(r_int_dir)/dlamch.f

lsame.o: $(r_int_dir)/lsame.f
	$(COMPILE.f90) $(r_int_dir)/lsame.f

besbod.o: $(r_bess_dir)/besbod.f
	$(COMPILE.f90) $(r_bess_dir)/besbod.f

besrat.o: $(r_bess_dir)/besrat.f
	$(COMPILE.f90) $(r_bess_dir)/besrat.f

bessrtra.o: $(r_bess_dir)/bessrtra.f
	$(COMPILE.f90) $(r_bess_dir)/bessrtra.f

euleracc.o: $(r_rut_dir)/euleracc.f
	$(COMPILE.f90) $(r_rut_dir)/euleracc.f

eimult.o: $(r_rut_dir)/eimult.f
	$(COMPILE.f90) $(r_rut_dir)/eimult.f

rhypsmall.o: $(r_rut_dir)/rhypsmall.f
	$(COMPILE.f90) $(r_rut_dir)/rhypsmall.f

hptrig.o: $(r_rut_dir)/hptrig.f
	$(COMPILE.f90) $(r_rut_dir)/hptrig.f

blanktoend.o: $(r_rut_dir)/blanktoend.f
	$(COMPILE.f90) $(r_rut_dir)/blanktoend.f

putintp.o: $(r_rut_dir)/putintp.f
	$(COMPILE.f90) $(r_rut_dir)/putintp.f

datan2dq.o: $(r_rut_dir)/datan2dq.f
	$(COMPILE.f90) $(r_rut_dir)/datan2dq.f

cdsqrt12.o: $(r_rut_dir)/cdsqrt12.f
	$(COMPILE.f90) $(r_rut_dir)/cdsqrt12.f

cfftal.o: $(r_rut_dir)/cfftal.f
	$(COMPILE.f90) $(r_rut_dir)/cfftal.f

silvia.o: $(r_rut_dir)/silvia.f
	$(COMPILE.f90) $(r_rut_dir)/silvia.f

zerodivy.o: $(r_rut_dir)/zerodivy.f
	$(COMPILE.f90) $(r_rut_dir)/zerodivy.f

argseg.o: $(r_rut_dir)/argseg.f
	$(COMPILE.f90) $(r_rut_dir)/argseg.f

hyprest.o: $(r_rut_dir)/hyprest.f
	$(COMPILE.f90) $(r_rut_dir)/hyprest.f

rtext.o: $(r_rut_dir)/rtext.f
	$(COMPILE.f90) $(r_rut_dir)/rtext.f

airy2e.o: $(r_slatec_dir)/airy2e.f
	$(COMPILE.f90) $(r_slatec_dir)/airy2e.f

airy4e.o: $(r_slatec_dir)/airy4e.f
	$(COMPILE.f90) $(r_slatec_dir)/airy4e.f

aizbiz.o: $(r_slatec_dir)/aizbiz.f\
                        d1mach.o
	$(COMPILE.f90) $(r_slatec_dir)/aizbiz.f

d1mach.o: $(linu_dir)/d1mach.f90
	$(COMPILE.f90) $(linu_dir)/d1mach.f90

#===============================================================

#--------------------
# Auxiliaries
#--------------------
clean:
	rm -rf *.o *.a *~ *.mod

