#===============================================================
#         Makefile for generating rmodfautSHf.exe
#
# Targets:
#         rmodfautSHf.exe, clean
#
# Usage:
#         To build rmodfaut using gfortran (GNU):
#            $ make -f rmodfaut.mk             ! only first target (default)
#         To clean up:
#            $ make -f rmodfaut.mk clean                       ! second target
#         To build rmodfaut using gfortran (GNU) and clean up:
#            $ make -f rmodfaut.mk rmodfautSHf.exe clean       ! both targets
#         To run the program:
#            $ chmod a+x rmodfautSHf.exe
#            $ ./rmodfautSHf.exe
#
#===============================================================

#--------------------
# Root directory
#--------------------
RP = /home/sveni/SI/oalib

#-------------------------
# Source code directories
#-------------------------
r_cpm_dir         = $(RP)/rpress/cpmQ
r_single_dir      = $(RP)/rpress/cpmQ/singleQ
r_dhcomp_dir      = $(RP)/rpress/cpmQ/dhcompQ
r_int_dir         = $(RP)/rpress/intQ
r_rut_dir         = $(RP)/rpress/rutQ
r_slatec_dir      = $(RP)/rpress/slatecQ

#-----------------------
# Lists of code to link
#-----------------------
r_cpm_list         = INMODELa.o PREDHASa.o GRLOCALa.o\
                     azimiln.o dhsubSHf.o ec.o fsubmodelshell.o\
                     harksi.o harksipart.o liqpart.o\
                     u2test.o corntestsub.o grderivSHf.o\
                     harkderiva.o harkderivb.o liqderivaSHf.o liqderivbSHf.o 
r_single_list      = liqsiSHf.o rmodelSHf.o
r_dhcomp_list      = dhcompSHf.o
r_int_list         = dpclin.o xpolbul.o dlamch.o lsame.o
r_rut_list         = dzeroholo.o sortxd.o\
                     eimult.o rhypsmall.o hptrig.o blanktoend.o putintp.o\
                     cdsqrt12.o\
                     argseg.o hyprest.o rtext.o qcmplx.o
r_slatec_list      = airy2e.o airy4e.o aizbiz.o

#--------------------
# Common code
#--------------------
OBJECTS = $(r_cpm_list) $(r_single_list) $(r_dhcomp_list)\
          $(r_int_list) $(r_rut_list) $(r_slatec_list)

#-----------------------------
# Compilers and compiler flags
#-----------------------------
F90 = /usr/bin/gfortran
FFLAGS  = -fcheck=all -fPIC -ffixed-line-length-132     -fno-align-commons
F_FLAGS =             -fPIC -ffixed-line-length-132 -O3 -fno-align-commons

#------------------
# Compile commands
#------------------
COMPILE.f90 = $(F90) $(FFLAGS) -c
COMPILE_f90 = $(F90) $(F_FLAGS) -c

#---------------
# Link commands
#---------------
LINK.f90 = $(F90) $(FFLAGS) -o
LINK_f90 = $(F90) $(F_FLAGS) -o

#===============================================================

#--------------------
# Link code
#--------------------
rmodfautSHf.exe: rmodfaut.o zerofind.o $(OBJECTS)
	$(LINK.f90) rmodfautSHf.exe rmodfaut.o zerofind.o $(OBJECTS)

#===============================================================

#--------------------
# Compile code
#--------------------
rmodfaut.o: $(r_cpm_dir)/rmodfaut.f\
                          INMODELa.o
	$(COMPILE.f90) $(r_cpm_dir)/rmodfaut.f

zerofind.o: $(r_rut_dir)/zerofind.f
	$(COMPILE.f90) $(r_rut_dir)/zerofind.f

#--------------------
# Compile code
#--------------------
INMODELa.o: $(r_cpm_dir)/INMODELa.f
	$(COMPILE.f90) $(r_cpm_dir)/INMODELa.f

PREDHASa.o: $(r_cpm_dir)/PREDHASa.f
	$(COMPILE.f90) $(r_cpm_dir)/PREDHASa.f

GRLOCALa.o: $(r_cpm_dir)/GRLOCALa.f
	$(COMPILE.f90) $(r_cpm_dir)/GRLOCALa.f

azimiln.o: $(r_cpm_dir)/azimiln.f
	$(COMPILE.f90) $(r_cpm_dir)/azimiln.f

dhsubSHf.o: $(r_cpm_dir)/dhsubSHf.f
	$(COMPILE.f90) $(r_cpm_dir)/dhsubSHf.f

ec.o: $(r_cpm_dir)/ec.f
	$(COMPILE.f90) $(r_cpm_dir)/ec.f

liqpart.o: $(r_cpm_dir)/liqpart.f
	$(COMPILE.f90) $(r_cpm_dir)/liqpart.f

u2test.o: $(r_cpm_dir)/u2test.f
	$(COMPILE.f90) $(r_cpm_dir)/u2test.f

corntestsub.o: $(r_cpm_dir)/corntestsub.f
	$(COMPILE.f90) $(r_cpm_dir)/corntestsub.f

grderivSHf.o: $(r_cpm_dir)/grderivSHf.f
	$(COMPILE.f90) $(r_cpm_dir)/grderivSHf.f

liqderivaSHf.o: $(r_cpm_dir)/liqderivaSHf.f
	$(COMPILE.f90) $(r_cpm_dir)/liqderivaSHf.f

liqderivbSHf.o: $(r_cpm_dir)/liqderivbSHf.f
	$(COMPILE.f90) $(r_cpm_dir)/liqderivbSHf.f

liqsiSHf.o: $(r_single_dir)/liqsiSHf.f
	$(COMPILE.f90) $(r_single_dir)/liqsiSHf.f

rmodelSHf.o: $(r_single_dir)/rmodelSHf.f
	$(COMPILE.f90) $(r_single_dir)/rmodelSHf.f

dhcompSHf.o: $(r_dhcomp_dir)/dhcompSHf.f
	$(COMPILE.f90) $(r_dhcomp_dir)/dhcompSHf.f

dpclin.o: $(r_int_dir)/dpclin.f
	$(COMPILE.f90) $(r_int_dir)/dpclin.f

xpolbul.o: $(r_int_dir)/xpolbul.f
	$(COMPILE.f90) $(r_int_dir)/xpolbul.f

dlamch.o: $(r_int_dir)/dlamch.f
	$(COMPILE.f90) $(r_int_dir)/dlamch.f

lsame.o: $(r_int_dir)/lsame.f
	$(COMPILE.f90) $(r_int_dir)/lsame.f

dzeroholo.o: $(r_rut_dir)/dzeroholo.f
	$(COMPILE.f90) $(r_rut_dir)/dzeroholo.f

sortxd.o: $(r_rut_dir)/sortxd.f
	$(COMPILE.f90) $(r_rut_dir)/sortxd.f

eimult.o: $(r_rut_dir)/eimult.f
	$(COMPILE.f90) $(r_rut_dir)/eimult.f

rhypsmall.o: $(r_rut_dir)/rhypsmall.f
	$(COMPILE.f90) $(r_rut_dir)/rhypsmall.f

hptrig.o: $(r_rut_dir)/hptrig.f
	$(COMPILE.f90) $(r_rut_dir)/hptrig.f

blanktoend.o: $(r_rut_dir)/blanktoend.f
	$(COMPILE.f90) $(r_rut_dir)/blanktoend.f

putintp.o: $(r_rut_dir)/putintp.f
	$(COMPILE.f90) $(r_rut_dir)/putintp.f

cdsqrt12.o: $(r_rut_dir)/cdsqrt12.f
	$(COMPILE.f90) $(r_rut_dir)/cdsqrt12.f

argseg.o: $(r_rut_dir)/argseg.f
	$(COMPILE.f90) $(r_rut_dir)/argseg.f

hyprest.o: $(r_rut_dir)/hyprest.f
	$(COMPILE.f90) $(r_rut_dir)/hyprest.f

rtext.o: $(r_rut_dir)/rtext.f
	$(COMPILE.f90) $(r_rut_dir)/rtext.f

qcmplx.o: $(r_rut_dir)/qcmplx.f
	$(COMPILE.f90) $(r_rut_dir)/qcmplx.f

airy2e.o: $(r_slatec_dir)/airy2e.f
	$(COMPILE.f90) $(r_slatec_dir)/airy2e.f

airy4e.o: $(r_slatec_dir)/airy4e.f
	$(COMPILE.f90) $(r_slatec_dir)/airy4e.f

aizbiz.o: $(r_slatec_dir)/aizbiz.f
	$(COMPILE.f90) $(r_slatec_dir)/aizbiz.f

#===============================================================

#--------------------
# Auxiliaries
#--------------------
clean:
	rm -rf *.o *.a *~ *.mod

